import os
import requests
from tqdm import tqdm

chunk_size = 1024

def download(url, directory):
	file_name = url.split('/')[-1]
	file_address = directory + '/' + file_name

	resp = requests.get(url, stream = True)
	try:
		total_size = int(resp.headers['content-length'])
	except KeyError:
		total_size = len(resp.content)

	total_chunks = total_size/chunk_size
	file_iterable = resp.iter_content(chunk_size = chunk_size)

	print("Downloading {}".format(file_name))
	
	with open(file_address, 'wb') as f:
		for data in tqdm(iterable = file_iterable, total = total_chunks, unit = 'KB'):
			f.write(data)

	print("{} has been saved to {}\n".format(file_name, file_address))


def download_all(urls, directory):
	if not os.path.exists(directory):
		os.makedirs(directory)

	for url in urls:
		download(url, directory)






FILE_EXTENSIONS = {  'Adobe Flash':'swf',
					 'Adobe Portable Document Format':'pdf',
					 'Adobe PostScript':'ps',
					 'Autodesk Design Web Format': 'dwf',
					 'Google Earth': 'kml',
					 'XML': 'xml',
					 'Microsoft PowerPoint': 'ppt',
					 'Microsoft Excel': 'xls',
					 'Microsoft Word': 'doc',
					 'GPS eXchange Format': 'gpx',
					 'Hancom Hanword': 'hwp',
					 'HTML': 'html',
					 'OpenOffice presentation': 'odp',
					 'OpenOffice spreadsheet': 'ods',
					 'OpenOffice text': 'odt',
					 'Rich Text Format': 'rtf',
					 'Scalable Vector Graphics': 'svg',
					 'TeX/LaTeX': 'tex',
					 'Text': 'txt',
					 'Basic source code': 'bas',
					 'C source code': 'c',
					 'C++ source code': 'cpp',
					 'C# source code': 'cs',
					 'Java source code': 'java',
					 'Perl source code': 'pl',
					 'Python source code': 'py',
					 'Wireless Markup Languag': 'wml'}


# -*- coding: utf-8 -*-
from setuptools import setup

def readme():
	try:
	    with open('README.rst') as f:
	        return f.read()
	except:
		pass

setup(name = 'ctdl',
      version = '1.0.0',
      classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
	    'Programming Language :: Python :: 2',
	    'Programming Language :: Python :: 2.6',
	    'Programming Language :: Python :: 2.7',
	    'Programming Language :: Python :: 3',
	    'Programming Language :: Python :: 3.3',
	    'Programming Language :: Python :: 3.4',
	    'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
      ],
      keywords = 'content downloader bulk files',
      description = 'Bulk file downloader on any topic.',
      long_description = readme(),
      url = 'http://github.com/nikhilkumarsingh/gnewsclient',
      author = 'Nikhil Kumar Singh',
      author_email = 'nikhilksingh97@gmail.com',
      license = 'MIT',
      packages = ['ctdl'],
      install_requires = ['requests', 'bs4', 'html5lib', 'tqdm'], 
      include_package_data = True,
      entry_points="""
      [console_scripts]
      ctdl = ctdl.ctdl:main
      """,
      zip_safe = False)


import argparse
import requests
from bs4 import BeautifulSoup
from .downloader import download_all
from .utils import FILE_EXTENSIONS

search_url = "https://www.google.com/search"


def scrape(html):
	soup = BeautifulSoup(html, 'html5lib')
	results = soup.findAll('h3', {'class': 'r'})
	links = []
	for result in results:
		link = result.a['href'][7:].split('&')[0]
		links.append(link)
	return links


def search(query, file_type = 'pdf', limit = 10):
	gquery = "filetype:{0} {1}".format(file_type, query)
	params = {
		'q': gquery,
		'start': 0,
	}

	headers = {
		'User Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:53.0) \
		               Gecko/20100101 Firefox/53.0'
	}

	links = []
	
	'''
	every Google search result page has a start index.
	every page contains 10 search results.
	'''
	for start_index in range(0, limit, 10):
		params['start'] = start_index
		resp = requests.get(search_url, params = params, headers = headers)
		page_links = scrape(resp.content)
		links.extend(page_links)

	return links[:limit]


def download_content(query, file_type = 'pdf', directory = None, limit = 10):
	if not directory:
		directory = query.replace(' ', '-')

	links = search(query, file_type, limit)
	download_all(links, directory)


def show_filetypes():
	for item in FILE_EXTENSIONS.items():
		print("{0:4}: {1}".format(item[1], item[0]))


def main():
    parser = argparse.ArgumentParser(description = "Content Downloader",
    								 epilog="Now download files on any topic in bulk!")
 
    # defining arguments for parser object
    parser.add_argument("query", type = str, default = None, nargs = '?',
    					help = "Specify the query.")

    parser.add_argument("-f", "--file_type", type = str, default = 'pdf',
                        help = "Specify the extension of files to download.")
     
    parser.add_argument("-l", "--limit", type = int, default = 10,
                        help = "Limit the number of search results (in multiples of 10).")
     
    parser.add_argument("-d", "--directory", type = str, default = None,
                        help = "Specify directory where files will be stored.")

    parser.add_argument("-a", "--available", action='store_true',
    					help = "Get list of all available filetypes.")
 
    args = parser.parse_args()
  
    if args.available:
    	show_filetypes()
    else:
    	download_content(args.query, args.file_type, args.directory, args.limit)


if __name__ == "__main__":
	main()
